//
//  QBChatModule+Typing.m
//  crossplatform-sdk
//
//  Created by Injoit on 26.12.2019.
//  Copyright © 2019 Injoit LTD. All rights reserved.
//

#import "QBChatModule+Typing.h"

#import <objc/runtime.h>

static void *_typingListeners;

@implementation QBChatModule (Typing)

- (NSMutableSet<QBDialogListener *> *)typingListeners {
    NSMutableSet<QBDialogListener *> *result = objc_getAssociatedObject(self,
                                                                        &_typingListeners);
    if (result == nil) {
        result = [NSMutableSet set];
        objc_setAssociatedObject(self,
                                 &_typingListeners,
                                 result,
                                 OBJC_ASSOCIATION_RETAIN_NONATOMIC);
    }
    return result;
}

- (void)sendIsTyping:(NSDictionary *)info
            resolver:(QBResolveBlock)resolve
            rejecter:(QBRejectBlock)reject {
    [self dialogWithInfo:info
                 success:^(QBChatDialog *dialog) {
        [dialog sendUserIsTyping];
        resolve(nil);
    } rejecter:reject];
}

- (void)sendStoppedTyping:(NSDictionary *)info
                 resolver:(QBResolveBlock)resolve
                 rejecter:(QBRejectBlock)reject {
    [self dialogWithInfo:info
                 success:^(QBChatDialog *dialog) {
        [dialog sendUserStoppedTyping];
        resolve(nil);
    } rejecter:reject];
}

- (void)subscribeTyping:(NSDictionary *)info
               resolver:(QBResolveBlock)resolve
               rejecter:(QBRejectBlock)reject {
    __weak __typeof(self)weakSelf = self;
    [self dialogWithInfo:info
                 success:^(QBChatDialog *dialog) {
        QBDialogListener *listner = [QBDialogListener new];
        listner.delegate = weakSelf;
        [listner subscribeWithDialog:dialog];
        [weakSelf.typingListeners addObject:listner];
        resolve(nil);
    } rejecter:reject];
}

- (void)unsubscribeTyping:(NSDictionary *)info
                 resolver:(QBResolveBlock)resolve
                 rejecter:(QBRejectBlock)reject {
    __weak __typeof(self)weakSelf = self;
    [self dialogWithInfo:info
                 success:^(QBChatDialog *dialog) {
        QBDialogListener *listner = [QBDialogListener new];
        listner.dialog = dialog;
        [weakSelf.typingListeners removeObject:listner];
        resolve(nil);
    } rejecter:reject];
}

@end

//MARK: DialogListnerProtocol
@implementation QBChatModule(DialogListnerTyping)

- (void)chatDidReciveIsTypingWithUserID:(NSNumber *)userID dialogID:(NSString *)dialogID {
    [self postQBEventWithName:QBUserTypingEvent.isTyping
                         body:@{ QBChatKey.userId: userID,
                                 QBChatKey.dialogId: dialogID }];
}

- (void)chatDidReciveStopTypingWithUserID:(NSNumber *)userID dialogID:(NSString *)dialogID {
    [self postQBEventWithName:QBUserTypingEvent.stopTyping
                         body:@{ QBChatKey.userId: userID,
                                 QBChatKey.dialogId: dialogID }];
}

@end
